import os
import pydicom
from pathlib import Path
from pydicom.tag import Tag
from collections import namedtuple
from pathlib import Path
import logging
from datetime import datetime


Filter = namedtuple('Filter', ['id', 'description', 'value', 'long_desc'])

now = datetime.today().strftime('%Y-%m-%d-%H-%M-%S')

root = Path(os.path.abspath(__file__)).parent
logpath = root / 'logs'
ANON_LOG_FILE = logpath / f"{now}.log"


if not os.path.exists(logpath):
    os.makedirs(logpath)

logging.basicConfig(filename=ANON_LOG_FILE,
                    filemode='a',
                    format='%(asctime)s %(levelname)s \t %(message)s',
                    datefmt='%H:%M:%S',
                    level=logging.INFO)


def generate_tags(user_list):
    tags = []
    for line in user_list:
        line = Filter(*line)
        tag = Tag([int(x, 16) for x in line.id.split(",")])  # Convert hex to int and get Tag
        tags.append(Filter(tag, line.description, line.value, line.long_desc))
    return tags


def load_dicom_file(filepath):
    try:
        return pydicom.dcmread(os.path.join(filepath))
    except pydicom.errors.InvalidDicomError:
        logging.error("Invalid DICOM")
        return None


def save_dicom_file(ds, savepath):
    savepath.parent.mkdir(parents=True, exist_ok=True)
    ds.save_as(str(savepath))
    print(f"Saving: {savepath}\n")
    logging.info(f"Saving: {savepath}\n\n")


def scrub_tags(ds, tags):
    for tag in tags:
        try:
            value_cur = ds[tag.id].value
            value_new = tag.value

            logging.info(f"{tag.id} {tag.description}: \t Found (replacing {value_cur} with {value_new})")
            ds[tag.id].value = value_new
        except KeyError:
            logging.error(f"{tag.id} {tag.description}: \t Not found (KeyError)")
        except AttributeError:
            logging.error(f"{tag.id} {tag.description}: \t Not found (AttributeError)")
        except TypeError:
            logging.error(f"{tag.id} {tag.description}: \t Not found (TypeError)")
        except:
            logging.error(f"{tag.id} {tag.description}: \t Unexpected error")

    return ds


def anonymise_file(source_filepath, dest_filepath, tags):
    f = Path(source_filepath)
    ds = load_dicom_file(source_filepath)

    if ds:
        print(f"Opening: {f}")
        logging.info(f"Opening: {f}")
        ds_anon = scrub_tags(ds, tags)

        # TODO handle better
        # savefile = os.path.join(f.parent, f"{f.stem}_anon.dcm")
        save_dicom_file(ds_anon, dest_filepath)



#! /usr/bin/python

import os, sys
import csv
import argparse
import dicom_anonymiser.tools as anon
from pathlib import Path

from gooey import Gooey, GooeyParser


root = Path(os.path.abspath(__file__)).parent
tagpath = root / 'tags'
default_tagfile = tagpath / 'default_tags.csv'
default_tagfile_text = str(default_tagfile)

@Gooey(program_name='DICOM Anonymiser',
       required_cols=1,
       optional_cols=1,
       default_size=(610, 630))
def parse_args():
    # parser = argparse.ArgumentParser(description='Anonymise DICOM images')

    parser = GooeyParser(description='Anonymise DICOM images')

    parser.add_argument('source', type=str, metavar="Source",
                        widget="DirChooser",
                        help='location of dicom file or folder to anonymise')

    parser.add_argument('destination', type=str, metavar="Destination",
                        widget="DirChooser",
                        help='Destination folder to save anonymised images')

    parser.add_argument('-t', '--tagfile', default=default_tagfile_text, metavar="Tag file",
                        widget="FileChooser",
                        help='path to custom tags file')

    parser.add_argument('-i', '--intact', action="store_true", metavar="Intact",
                        help='Leave filenames unchanged')



    args = parser.parse_args()

    args.tagfile = Path(args.tagfile)
    args.source = Path(args.source)
    args.destination = Path(args.destination)

    if (args.source == args.destination) and (args.intact_filename):
        print('Source and destination are the same and filenames to be left intact. This will overwrite original files. Exiting.')
        sys.exit()
    return args


def tag_list(tagfile):
    data_elements = []
    with open(tagfile) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        for row in csv_reader:
            data_elements.append(row)

    return anon.generate_tags(data_elements)


def file_rename(filename, keep_intact):
    if not keep_intact:
        return filename.replace('.dcm', '_anon.dcm', 1)
    else:
        return filename


def get_dest_filepath(src, src_root, dst_root):
    """
    This function will build the full abs path of the destination file
    :param src: The full abs path to the file to be anonymised
    :param src_root: The top level source directory
    :param dst_root: The top level destination directory
    :return: full destination file abs path
    """

    src_parts = list(src.parts)  # split the source into parts
    src_parts = src_parts[len(src_root.parts):]  # truncate the source root
    return Path(*list(dst_root.parts), *src_parts)  # add the destination root


def main():
    args = parse_args()
    my_tags = tag_list(args.tagfile)

    if args.source.is_dir():
        for root, dirs, files in os.walk(args.source):
            for file in files:
                source_filepath = Path(root) / file
                dest_filename = file_rename(file, args.intact)
                dest_filepath = get_dest_filepath(Path(root) / dest_filename, args.source, args.destination)
                anon.anonymise_file(source_filepath, dest_filepath, my_tags)
    else:
        file = file_rename(args.source.name, args.intact)
        dest_filepath = args.destination / file
        anon.anonymise_file(args.source, dest_filepath, my_tags)


if __name__ == '__main__':
    main()



"""dicom_anonymiser setup"""

import setuptools


with open('README.md') as f:
    README = f.read()

setuptools.setup(
    author="Eric Pace",
    author_email="ericpace@pm.me",
    name='dicom_anonymiser',
    license="GNU GPLv3",
    description='dicom_anonymiser anonymises dicom files and folders with user customisable tags.',
    version='v0.1',
    long_description='dicom_anonymiser anonymises dicom files and folders with user customisable tags.',
    url='https://github.com/ericpace/dicom_anon_tool',
    packages=setuptools.find_packages(),
    include_package_data=True,
    entry_points={'gui_scripts': ['anon=dicom_anonymiser.__main__:main']},
    python_requires=">=3.5",
    # Enable install requires when publishing on the normal PyPi
    install_requires=[
        'Gooey',
        'pydicom'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
    ],
)